' John Conway's Game of Life for MMBasic
'
' Suitable for:
' The Micromite MkII
' The CMM
' The CMM2 (hopefully)
' And MMBasic for DOS
'
' Bill McKinley
' 
DV = 0      ' Device type
MatX = 40   ' Matrix horizontal size
MatY = 25   ' Matrix vertical size
RandL = 0.3 ' For the randomizing of life less than =< RandL means there is life
            ' Make it high and you get a lot of cells but they die quickly
' The Delay in mS between display updates
dim PT(4) = (0, 0, 300, 0, 600) ' 0, CMM, CMM2, Micromite MkII, DOS

dim AA$(MatX)
' Zero based matrix has one more cell all round than is displayed
dim CurM(MatX+1, MatY+1, 1) ' The current matrix of life - twice
Who         ' Find out what device is trying to run this
Restart:
ClrMat      ' Clear the matrix if restarting
ChkScr      ' Check the window size and print intro
SetupTimer  ' Randomize by waiting for a keypress
ClrScr      ' Clear the screen (matrix area)
InitM       ' Initialize the random matrix (or demo)
g2 = 0
do          ' Program loop
G2 = 1 - G2 ' Changes 0>1>0... each time around
G1 = 0 = G2
NextGen     ' Calculate the next generation
pause PT(DV)
E$ = inkey$
loop until E$ <> "" ' loop forever or until a keypres
if E$ = "R" or E$ = "r" then goto Restart
end

sub ChkScr
if DV = 4 then  ' Check the DOS window size
  w2=0
  do
  if MatX > mm.hres or MatY > mm.vres then 'Check the size of the DOS window
    w=0
    if w2=0 then 'Not printed the window size message yet
      print " Please increase the"
      print " Window size to see"
      print "  full matrix"
    w2=1
    endif
  else
    w=1
  endif
  loop until w=1
endif
ClrScr
print " CONWAY'S GAME OF LIFE"
print space$(20)
print " Press a key random start"
PRINT " 0 diehard"
PRINT " 1 glider"
PRINT " 2 blinker"
PRINT " 3 toad"
PRINT " 4 beacon"
PRINT " 5 Penta-decathlon"
PRINT " 6 pulsar"
PRINT " 7 LW spaceship"
PRINT " 8 MW spaceship"
PRINT " 9 HW spaceship"
print " R to restart"
print " any other to stop"
end sub

sub SetupTimer ' Wait for a key press to randomize
Do
  E$ = inkey$
loop until E$ <> ""
randomize timer
end sub

sub ClrScr 'Clear the screen area appropriate to device
if DV = 4 then
    cursor 0,0
  for y=1 to MatY
    print space$(MatX)
  next y
    cursor 0,0
elseif DV=0 or DV=1 then
  cls
elseif DV=3 then
PRINT CHR$(27) "[2J" CHR$(27) "[H"
endif
end sub

sub InitM ' Initialise the matrix of life
restore
E1 = -1 : E2 = 0
if E$ => "0" and E$ =< "9" then
  E1 = val(E$)
endif
if E1 = -1 then
  for y=1 to MatY
    for x=1 to MatX
      if rnd() =< RandL then  CurM(x,y,0) = 1
    next x
  next y
else ' we need to read a set configuration
  if E1 <> E2 then ' Skip over configs
    do
      do
        READ x
        READ y
      loop until x = -1
      E2 = E2 + 1
    loop until E1= E2 ' Got the right one
  endif
  DO
    READ x
    READ y
    IF x = -1 THEN EXIT DO
    IF y = -1 THEN PRINT "Error in Data",x,y: EXIT DO
    CurM(x, y, b) = 1
  LOOP
ENDIF
end sub

sub ClrMat ' zero the initial matrix
for y=1 to MatY
  for x=1 to MatX
    CurM(x,y,0) = 0
  next x
next y
end sub

' Removed white space from this sub to (hopefully) gain a few uS
' Breed the next generation into the alternate matrix
Sub NextGen
 For y=1 To MatY
   b=0
   c=CurM(1,y-1,g1)+CurM(1,y,g1)+CurM(1,y+1,g1)
   S$=""
   For x=1 To MatX
     d = CurM(x+1,y-1,g1)+CurM(x+1,y,g1)+CurM(x+1,y+1,g1)
     adj=b+c+d
     If CurM(x,y,g1) Then
       S$=S$+"O"
       CurM(x,y,g2)=(adj=3) Or (adj=4)
     Else
       CurM(x,y,g2)=(adj=3)
       S$=S$+" "
     EndIf
     b=c
     c=d
   Next x
     aa$(y)=s$
 Next y
if DV=4 then
  cursor 0,0
elseif DV=0 or DV=1 then
  print @(0,0)
elseif DV=3 then
  print CHR$(27) "[H"
endif
for i=1 to MatY:print AA$(i):next i
end sub

sub Who ' What device is running this program
f$ = mm.device$
IF f$ = "Colour Maximite" then
  DV = 1
elseif f$ = "Colour Maximite 2" then
  DV = 2
elseif f$ = "Micromite MkII" then
  DV = 3
elseif f$ = "DOS" then
  DV = 4
endif ' Defaults to DV = 0
end sub

 ' seeds are pairs of x,y cells ending in -1 then 0 or 1
seed0:  'diehard
'DATA 8, 5,2,6,3,6,4,6,7,7, 8,7, 9,7,-1,1
 DATA 12,5,6,6,7,6,7,7,11,7,12,7,13,7,-1,1
seed1: ' glider
 DATA 4,7,5,7,6,7,6,6,5,5,15,15,15,16,16,15,16,16,-1,0
seed2: ' blinker
 DATA 4,7,5,7,6,7,-1,0
seed3: ' toad
 DATA 5,7,6,7,7,7,4,8,5,8,6,8,-1,0
seed4: ' beacon
 DATA 4,7,5,7,4,8,7,9,6,10,7,10,-1,0
seed5: 'Penta-decathlon
 DATA 5, 7,6, 7,7, 7,5, 8,7, 8,5, 9,6, 9,7, 9
 DATA 5, 10,6, 10,7, 10,5, 11,6, 11,7, 11,5, 12
 DATA 6, 12,7, 12,5, 13,7, 13,5, 14,6, 14,7, 14,-1,1
seed6: ' pulsar
 DATA 5,3,6,3,7,3,11,3,12,3,13,3,3,5,8,5,10,5,15,5
 DATA 3,6,8,6,10,6,15,6,3,7,8,7,10,7,15,7
 DATA 5,8,6,8,7,8,11,8,12,8,13,8,5,10,6,10,7,10,11,10,12,10,13,10
 DATA 3,11,8,11,10,11,15,11,3,12,8,12,10,12,15,12
 DATA 3,13,8,13,10,13,15,13,5,15,6,15,7,15,11,15,12,15,13,15
 DATA -1, 1
seed7: ' LW spaceship
 DATA 3,2,6,2,7,3,3,4,7,4,4,5,5,5,6,5,7,5,-1,1
seed8: ' MW spaceship
 DATA 5,2,3,3,7,3,8,4,3,5,8,5,4,6,5,6,6,6,7,6,8,6,-1,1
seed9: ' HW spaceship
 DATA 7,4,8,4,3,5,4,5,5,5,6,5,8,5,9,5,3,6,4,6,5,6,6,6,7,6,8,6
 DATA 4,7,5,7,6,7,7,7,-1,1
  